
//uniform sampler2D SamplerRGB;
uniform sampler2D lowDetailSamplerRGB;
uniform sampler2D edgeSamplerRGB;
uniform highp vec2 backbufferSize;
uniform sampler2D lutTex;
uniform highp float edgeSize;

varying highp vec2 texCoord;


#+LUT_BLITTER
  
const mediump float loopsize = 90.0;
const mediump vec2 mul15deg=vec2(0.96592582,0.25881904);
const mediump vec2 mul45deg=vec2(0.70710678,0.70710678);

  
mediump float pattern(mediump vec2 p)
{
	return clamp(1.3*distance(fract(p), vec2(0.5,0.5)), 0.1, 1.0);
}


void main()
{
	mediump vec3 bg = texture2D(lowDetailSamplerRGB, texCoord).rgb;
	
	// Convert to CMYK
	mediump vec4 cmyk;
	cmyk.w = min( min( 1.0-bg.r, 1.0-bg.g), 1.0-bg.b );
	cmyk.x = (1.0 - bg.r - cmyk.w) / (1.0-cmyk.w);
	cmyk.y = (1.0 - bg.g - cmyk.w) / (1.0-cmyk.w);
	cmyk.z = (1.0 - bg.b - cmyk.w) / (1.0-cmyk.w);
	
	// contrast?
	cmyk = (cmyk - vec4(0.5,0.5,0.5,0.5))*1.3 + vec4(0.5,0.5,0.5,0.5);
	
	mediump vec2 tc = vec2( texCoord.x, texCoord.y*(backbufferSize.y/backbufferSize.x)) * loopsize;

	mediump vec4 rasterSample = vec4( pattern(vec2( tc.x * mul15deg.x + tc.y * mul15deg.y,tc.x * mul15deg.y + tc.y * -mul15deg.x )),
									  pattern(vec2( tc.x * mul15deg.y + tc.y * mul15deg.x,tc.x * mul15deg.x + tc.y * -mul15deg.y )),
									  pattern(tc ),
									  pattern(vec2( tc.x * mul45deg.x + tc.y * mul45deg.y, tc.x * mul45deg.y + tc.y * -mul45deg.x )));
													
	
	cmyk = clamp( smoothstep(0.0, 17.0, vec4(0.5, 0.5, 0.5, 0.5) + (cmyk-rasterSample)*15.0)*17.0, 0.0, 1.0);
	
	//mediump vec3 edgesample = texture2D( edgeSamplerRGB, texCoord ).rgb - bg;
	//mediump float edgepower = clamp((length( edgesample )-0.05)*10.0, 0.0, 1.0);
	
	// bg back to rgb
	bg.r = 1.0 - min(1.0,cmyk.x * (1.0-cmyk.w) + cmyk.w);
	bg.g = 1.0 - min(1.0,cmyk.y * (1.0-cmyk.w) + cmyk.w);
	bg.b = 1.0 - min(1.0,cmyk.z * (1.0-cmyk.w) + cmyk.w);
	

	//gl_FragColor = vec4( bg, 1.0);
	

	mediump vec2 samplingUnit = vec2( edgeSize/backbufferSize.x, edgeSize/backbufferSize.y );
	mediump vec3 topleft = texture2D(edgeSamplerRGB, texCoord + vec2(-samplingUnit.x, -samplingUnit.y)).rgb;
	mediump vec3 topmiddle = texture2D(edgeSamplerRGB, texCoord + vec2(0.0, -samplingUnit.y)).rgb;
	mediump vec3 topright = texture2D(edgeSamplerRGB, texCoord + vec2(samplingUnit.x, -samplingUnit.y)).rgb;
	mediump vec3 middleleft = texture2D(edgeSamplerRGB, texCoord + vec2(-samplingUnit.x, 0.0)).rgb;
	mediump vec3 middleright = texture2D(edgeSamplerRGB, texCoord + vec2(samplingUnit.x, 0.0)).rgb;
	mediump vec3 bottomleft = texture2D(edgeSamplerRGB, texCoord + vec2(-samplingUnit.x, samplingUnit.y)).rgb;
	mediump vec3 bottommiddle = texture2D(edgeSamplerRGB, texCoord + vec2(0.0, samplingUnit.y)).rgb;
	mediump vec3 bottomright = texture2D(edgeSamplerRGB, texCoord + vec2(samplingUnit.x, samplingUnit.y)).rgb;


	mediump vec3 gX = -topleft + topright + -2.0*middleleft + 2.0*middleright + -bottomleft + bottomright;
	mediump vec3 gY = -topleft + -2.0*topmiddle -topleft + bottomleft + 2.0*bottommiddle + bottomright;
	mediump float edgepower = (sqrt(dot(gX,gX) + dot(gY,gY)) - 0.2) * 30.0;
/*
	gl_FragColor = mix( vec4(bg,1.0), 
					    vec4(0.0, 0.0, 0.0, 1.0), 
					    clamp(edgepower,0.0,1.0) );
 */
    gl_FragColor = mix( lut3d(bg),
                       vec4(0.0, 0.0, 0.0, 1.0),
                       clamp(edgepower,0.0,1.0) );
}