uniform sampler2D SamplerRGB;
uniform highp vec2 backbufferSize;
uniform highp float loopsize;

varying highp vec2 texCoord;

void main (void)
{
    highp vec2 pmul = vec2( loopsize, loopsize/backbufferSize.x*backbufferSize.y );
    highp vec2 pos = texCoord * pmul;
	
    
	// 	skew
	highp float F2 = 0.3660254037844386; // 0.5*(sqrt(3.0)-1.0);
	highp float s = (pos.x + pos.y)*F2;
	highp vec2 ij=vec2(floor( pos.x + s ), floor(pos.y + s ) );
	
	
	highp float G2 = 0.2113248654051871; //(3.0 - sqrt(3.0))/6.0;
	highp float t = (ij.x+ij.y)*G2;
	// UNSKEWED BACK!
	highp vec2 pos0 = vec2(ij.x-t,ij.y-t);
	
	highp vec2 xy0 = pos - pos0;
	highp vec2 ij1;
	if (xy0.x>xy0.y) ij1 = vec2(1,0); else ij1 = vec2(0,1);
		
	highp vec2 xy1 = xy0 - ij1 + vec2(G2,G2);
	highp vec2 xy2 = xy0 - vec2(1,1) + 2.0 * vec2(G2, G2);
	

    /*
    // try direct blending?
	highp float t0 = dot(xy0, xy0);
	t0 = clamp( 0.5 - t0*t0, 0.0, 1.0 );
	
	highp float t1 = dot(xy1, xy1);
	t1 = clamp( 0.5 - t1*t1, 0.0, 1.0 );
	
	highp float t2 = dot(xy2, xy2);
	t2 = clamp( 0.5 - t2*t2, 0.0, 1.0 );
     */

    

    highp float t0 = dot(xy0, xy0);
    t0 = clamp( 0.75 - pow(t0,0.2), 0.0, 1.0 );
    
    highp float t1 = dot(xy1, xy1);
    t1 = clamp( 0.75 - pow(t1,0.2), 0.0, 1.0 );
    
    highp float t2 = dot(xy2, xy2);
    t2 = clamp( 0.75 - pow(t2,0.2), 0.0, 1.0 );


    /*
    // try direct blending?
    mediump float t0 = dot(xy0, xy0);
    t0 = clamp( 0.5 - t0, 0.0, 1.0 );
    mediump float t1 = dot(xy1, xy1);
    t1 = clamp( 0.5 - t1, 0.0, 1.0 );
    mediump float t2 = dot(xy2, xy2);
    t2 = clamp( 0.5 - t2, 0.0, 1.0 );
*/


	
	highp float tot = t0+t1+t2;
	highp vec3 tcol = texture2D( SamplerRGB, pos0/pmul ).rgb * t0 / tot +
                        texture2D( SamplerRGB, (pos-xy1)/pmul ).rgb * t1 / tot +
						texture2D( SamplerRGB, (pos-xy2)/pmul ).rgb * t2 / tot;
	
    gl_FragColor = vec4(tcol,1.0);
}
